
//********************************************************************
//	created:	2006-1-16  16:13
//	filename: 	Vector3D.cpp
//	file base:	Vector3D
//	author:		пԺȽоԺ Ӽ (Liangliang Nan)
//  E-mail:     liangliang.nan@gmail.com
// 
//	
//	purpose:	implementation of the CVector3D class.
//*********************************************************************

#include "stdafx.h"
#include "Vector3D.h"
#include "Point3D.h"
#include <cmath>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVector3D::CVector3D()
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
}


CVector3D::CVector3D(float fx, float fy, float fz)
{
	x = fx;
	y = fy;
	z = fz;
}


CVector3D::CVector3D(const float* pv)
{
	x = pv[0];
	y = pv[1];
	z = pv[2];
}


CVector3D::CVector3D(const CVector3D& v)
{	
	x = v.x;
	y = v.y;
	z = v.z;
}

CVector3D::CVector3D(CPoint3D source, CPoint3D dest)
{
	x = dest.x - source.x;
	y = dest.y - source.y;
	z = dest.z - source.z;
}

CVector3D::~CVector3D()
{
}


CVector3D CVector3D::operator +(CVector3D v) const
{
	return CVector3D(x + v.x, y + v.y, z + v.z);
}


CVector3D CVector3D::operator -(CVector3D v) const
{
	return CVector3D(x - v.x, y - v.y, z - v.z);
}


void CVector3D::operator +=(CVector3D v)
{
	x += v.x;
	y += v.y;
	z += v.z;
}


void CVector3D::operator -=(CVector3D v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
}


CVector3D CVector3D::operator *(float d) const
{
	return CVector3D(x * d, y * d, z * d);
}


void CVector3D::operator *=(float d)
{
	x *= d;
	y *= d;
	z *= d;
}


CVector3D CVector3D::operator /(float d) const
{
	return CVector3D(x / d, y / d, z / d);
}

	
void CVector3D::operator /=(float d)
{
	x /= d;
	y /= d;
	z /= d;
}


// cross product
CVector3D CVector3D::operator *(CVector3D v) const
{
	return CVector3D(y * v.z - v.y * z, 
					 z * v.x - x * v.z,
					 x * v.y - y * v.x);
}

	
// dot product
float CVector3D::operator |(CVector3D v) const
{
	return x * v.x + y * v.y + z * v.z;
}

float CVector3D::DotProduct(CVector3D v) const
{
	return x * v.x + y * v.y + z * v.z;
}

//methods implementation
float CVector3D::GetLength() const
{
	return sqrtf(x*x + y*y + z*z);
}

float CVector3D::GetSquaredLength() const
{
	return (x*x + y*y + z*z);
}

float CVector3D::GetLengthXY() const
{
	return sqrtf(x*x + y*y);
}


float CVector3D::GetLengthYZ() const
{
	return sqrtf(y*y + z*z);
}


float CVector3D::GetLengthZX() const
{
	return sqrtf(x*x + z*z);
}


CVector3D CVector3D::GetNormal() const
{
	float len = GetLength();
	return CVector3D(x/len, y/len, z/len);
}


void CVector3D::Normalize()
{
	float len = GetLength();
	if (len != 0)
	{
		x /= len;
		y /= len;
		z /= len;
	}
}


