//********************************************************************
//	created:	2006-1-13  9:56
//	filename: 	OpenGLView.h
//	file base:	OpenGLView
//	author:		пԺȽоԺ Ӽ (Liangliang Nan)
//  E-mail:     liangliang.nan@gmail.com
// 
//	
//	purpose:	interface for the COpenGLView class.
//*********************************************************************

#if !defined _OPENGLVIEW_H_
#define _OPENGLVIEW_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <gl/glut.h>

/*///////////////////////////////////////////////////////////////

  This class is a base view-class for simple OpenGL-programming
  Applications should subclass this class to get a working view.
  
	This class will draw a yellow sphere on a black background
	if your subclass does not redefine any of the functionality
	_____________________________________
	
	  The class handles the following:
	  1. Sets up the drawing environment;
	  2. Do one-time initialization of OpenGL-rendering context;
	  3. Draws the actual scene;
	  4. Handles some key messages (see below)
	  _____________________________
	  
		Handling following messages:
		1. OnSize(), Redefined in most cases, the viewing pyramid, parallell perspective, front and back clip;
		2. OnCreate(), usually not redefined;
		3. OnEraseBkgnd(), seldom (never) redefined;
		4. OnDestroy(), usually not redefined. Implementations in subclass must include a call to COpenGLView::OnDestroy();
		5. PreCreateWindow(), seldom (never) redefined;
		6. OnDraw(), seldom (never) redefined.
		________________________________
		
		  Default funcions
		  1. InitializeRContext(), virtual function. Allmost allways redefined.
		  Deals with lighting, back color, shading model etc. 
		  Things that will usually be done once in the programs lifetime.
		  2. RenderScene(), virtual function. Allways redefined
		  _________________________________
		  
		ı䱳ɫ,µĴ(ΪglClearColorۺʱ):
		void CREModelingView::OnOptionColorBackground() 
		{
			// TODO: Add your command handler code here
			CColorDialog BKColorDlg;
			BKColorDlg.m_cc.Flags |= CC_FULLOPEN;
              
		    if (BKColorDlg.DoModal() == IDOK)
			{
			  	  COLORREF Color = BKColorDlg.GetColor();
				  SetBackGroundColor(Color);
			}
		}
				  
*/
/////////////////////////////////////////////////////////////////////////////

#define NUM_MATERIALS 28

static enum {
		EMERALD = 0, 
		JADE,    
		OBSIDIAN, 
		PEARL,
		RUBY,
		TURQUOISE,
		BRASS,
		BRONZE,
		CHROME,
		COPPER,
		GOLD,
		SILVER,
		BLACK_PLASTIC,
		CYAN_PLASTIC,
		GREEN_PLASTIC,
		RED_PLASTIC,
		WHITE_PLASTIC,
		YELLOW_PLASTIC,
		BLACK_RUBBER,
		CYAN_RUBBER,
		GREEN_RUBBER,
		RED_RUBBER,
		WHITE_RUBBER,
		YELLOW_RUBBER,
		BRIGHT_WHITE,
		LESS_BRIGHT_WHITE,
		WARM_WHITE,
		COOL_WHITE
};


static GLfloat Material[NUM_MATERIALS][13] = {
	//EMERALD
	0.0215f, 0.1745f, 0.0215f,1.0f,
        0.07568f, 0.61424f, 0.07568f, 1.0f,
		0.633f, 0.727811f, 0.633f, 1.0f,
		0.6f,
		
		//JADE
		0.135f, 0.2225f, 0.1575f,1.0f,
        0.54f, 0.89f, 0.63f, 1.0f,
		0.316228f, 0.316228f, 0.316228f, 1.0f,
		0.1f,
		//OBSIDIAN
		0.05375f, 0.05f, 0.06625f, 1.0f,// XX
        0.18275f, 0.17f, 0.22525f, 1.0f,
		0.332741f, 0.328634f, 0.346435f,1.0f, 
		0.3f,
		//PEARL
		0.25f, 0.20725f, 0.20725f,1.0f,
        1.0f, 0.829f, 0.829f, 1.0f,
		0.296648f, 0.296648f, 0.296648f, 1.0f,
		0.088f,
		//RUBY
		0.1745f, 0.01175f, 0.01175f,1.0f,
        0.61424f, 0.04136f, 0.04136f, 1.0f,
		0.727811f, 0.626959f, 0.626959f, 1.0f,
		0.6f,
		//TUQUOISE
		0.1f, 0.18725f, 0.1745f,1.0f,
        0.396f, 0.74151f, 0.69102f, 1.0f,
		0.297254f, 0.30829f, 0.306678f, 1.0f,
		0.1f,
		//BRASS
		0.329412f, 0.223529f, 0.027451f,1.0f,
        0.780392f, 0.568627f, 0.113725f, 1.0f,
		0.992157f, 0.941176f, 0.807843f,1.0f,
        0.21794872f,
		//BRONZE
		0.2125f, 0.1275f, 0.054f,1.0f,
        0.714f, 0.4284f, 0.18144f, 1.0f,
		0.393548f, 0.271906f, 0.166721f, 1.0f,
		0.2f,
		//CHROME
		0.25f, 0.25f, 0.25f, 1.0f, // XX
        0.4f, 0.4f, 0.4f, 1.0f,
		0.774597f, 0.774597f, 0.774597f, 1.0f,
		0.6f,
		//COPPER
		0.19125f, 0.0735f, 0.0225f,1.0f,
        0.7038f, 0.27048f, 0.0828f, 1.0f,
		0.256777f, 0.137622f, 0.086014f, 1.0f,
		0.1f,
		//GOLD
		0.24725f, 0.1995f, 0.0745f,1.0f,
        0.75164f, 0.60648f, 0.22648f, 1.0f,
		0.628281f, 0.555802f, 0.366065f, 1.0f,
		0.4f,
		//SILVER
		0.19225f, 0.19225f, 0.19225f,1.0f,
        0.50754f, 0.50754f, 0.50754f, 1.0f,
		0.508273f, 0.508273f, 0.508273f, 1.0f,
		0.4f,
		//BLACK PLASTIC
		0.0f, 0.0f, 0.0f, 1.0f,
		0.01f, 0.01f, 0.01f,1.0f,
        0.50f, 0.50f, 0.50f, 1.0f,
		.25f,
		//CYAN PLASTIC
		0.0f, 0.1f, 0.06f, 0.0f, 1.0f,
		0.50980392f, 0.50980392f,1.0f,
        0.50196078f, 0.50196078f, 1.0f,
		0.50196078f, 
		.25f,
		//GREEN PLASTIC
		0.0f, 0.0f, 0.0f,1.0f,
        0.1f, 0.35f, 0.1f, 1.0f,
		0.45f, 0.55f, 0.45f, 1.0f,
		.25f,
		//RED PLASTIC
		0.0f, 0.0f, 0.0f, 1.0f,
		0.5f, 0.0f, 0.0f, 1.0f,// XX
        0.7f, 0.6f, 0.6f, 1.0f,
		.25f,
		//WHITE PLASTIC
		0.0f, 0.0f, 0.0f, 1.0f,
		0.55f, 0.55f, 0.55f,1.0f,
        0.70f, 0.70f, 0.70f, 1.0f,
		.25f,
		//YELLOW PLASTIC
		0.0f, 0.0f, 0.0f, 1.0f,
		0.5f, 0.5f, 0.0f,1.0f,
        0.60f, 0.60f, 0.50f, 1.0f,
		.25f,
		//BLACK RUBBER
		0.02f, 0.02f, 0.02f, 1.0f,
		0.01f, 0.01f, 0.01f, 1.0f,// XX
        0.4f, 0.4f, 0.4f, 1.0f,
		.078125f,
		//CYAN RUBBER
		0.0f, 0.05f, 0.05f, 1.0f,
		0.4f, 0.5f, 0.5f,1.0f,
        0.04f, 0.7f, 0.7f, 1.0f,
		.078125f,
		//GREEN RUBBER
		0.0f, 0.05f, 0.0f, 1.0f,
		0.4f, 0.5f, 0.4f,1.0f,
        0.04f, 0.7f, 0.04f, 1.0f,
		.078125f,
		//RED RUBBER
		0.05f, 0.0f, 0.0f, 1.0f,
		0.5f, 0.4f, 0.4f,1.0f,
        0.7f, 0.04f, 0.04f, 1.0f,
		.078125f,
		//WHITE RUBBER
		0.05f, 0.05f, 0.05f, 1.0f,
		0.5f, 0.5f, 0.5f,1.0f,
        0.7f, 0.7f, 0.7f, 1.0f,
		.078125f,
		//YELLOW RUBBER
		0.05f, 0.05f, 0.0f, 1.0f,
		0.5f, 0.5f, 0.4f,1.0f,
        0.7f, 0.7f, 0.04f, 1.0f,
		.078125f,
		// bright white
		0.2f, 0.2f, 0.2f,1.0f,
        1.0f, 1.0f, 1.0f, 1.0f,
		0.8f, 0.8f, 0.8f, 1.0f,
		0.4f,
		// less bright white
		0.2f, 0.2f, 0.2f,1.0f,
        0.8f, 0.8f, 0.8f, 1.0f,
		0.5f, 0.5f, 0.5f, 1.0f,
		0.35f,
		// warmish white
		0.3f, 0.2f, 0.2f,1.0f,
        1.0f, 0.9f, 0.8f, 1.0f,
		0.4f, 0.2f, 0.2f, 1.0f,
		0.35f,
		// coolish white
		0.2f, 0.2f, 0.3f,1.0f,
        0.8f, 0.9f, 1.0f, 1.0f,
		0.2f, 0.2f, 0.4f, 1.0f,
		0.35f
};

// COpenGLView view
class AFX_EXT_CLASS COpenGLView : public CView
{
	
public:
	//Ľӿ
	virtual void RenderSceneScalable();			// ƿŵ
	virtual void RenderSceneUnScalable();		// Ʋŵ壬 

	void  SetBackGroundColor(COLORREF color);   // ñɫ
	void  EnableLighting(bool b);               // Ƿ

	// ò
	void  SetMaterial(UINT nMatIndex = 10, GLenum gSide = GL_FRONT_AND_BACK); 

protected:
	GLuint  m_nAxisesList;					// ʾб	

	int     m_nFontOffset;					// ע"X, Y, Z"	
	GLenum  m_enDisplayMode;				// ߿򣬶㣬ʵ壿

	int     m_nCurrentMatIndex;				// material index	
	bool    m_bEnableLighting;				// Ƿù	
	bool    m_bEnableBothSidesLights;       // Ƿ˫	
	bool    m_bEnableAntiAliasing;          // Ƿ÷
	bool    m_bShowCoordinate;				// Ƿʾ

	bool    m_bLeftButtonDown;
	bool    m_bRightButtonDown;
	CPoint  m_ptLeftPrevDownPos;
	CPoint  m_ptRightPrevDownPos;
	
	CPoint  m_ptZoomSelectOrigin;			// ʱµ

	bool    m_bXZRotation;	
	GLfloat m_xRotation;
	GLfloat m_yRotation;
	GLfloat m_zRotation;

	GLfloat m_Scaling;
	GLfloat m_xTranslation;
	GLfloat m_yTranslation;
	GLfloat m_zTranslation;

	// FitWindowʱʹ
	GLboolean m_bLockScaleAndTrans;
	GLfloat m_stepScaling;
	GLfloat m_stepTransX; 
	GLfloat m_stepTransY; 
	GLfloat m_stepTransZ; 

	GLfloat m_SpeedScaling;
	GLfloat m_SpeedTranslation;
	GLfloat m_SpeedRotation;	
	
	GLfloat m_StepRotationX;				// ת;
	GLfloat m_StepRotationY;
	GLfloat m_StepRotationZ;
	
	GLfloat m_OrthoRangeLeft;				// ƽͶӰ
	GLfloat m_OrthoRangeRight;
	GLfloat m_OrthoRangeTop;
	GLfloat m_OrthoRangeBottom;
	GLfloat m_OrthoRangeNear;
	GLfloat m_OrthoRangeFar;


private:
	HGLRC      m_hRC;						// rendering context handle

	GLfloat m_BackColorRed;					// ɫ;
	GLfloat m_BackColorGreen;
	GLfloat m_BackColorBlue;
	GLfloat m_BackAlpha;	


	// Operations
protected:
	void InitRenderContext();

	BOOL SetWindowPixelFormat(HDC hDC);     // SetWindowPixelFormat

	//********************************************
	// CreateViewGLContext
	// Create an OpenGL rendering context
	//********************************************
	BOOL CreateViewGLContext(HDC hDC);

	void PrintAxisLabel(const char* str);	 // ע"X, Y, Z"ĸ
	void PrepareAxisLabel();
	
	void BuildAxisesList();					 // ʾб

	void InitGeometry();					 // ʼģ͵ʾ

	void ViewFitWindow();					 // ʹģʹСʺϴ

	void ViewPosTop();
	void ViewPosBottom();
	void ViewPosFront();
	void ViewPosBack();
	void ViewPosLeft();
	void ViewPosRight();
	void ViewPosTrimetric();

	void ViewLight();                             
	void ViewLightBothSides();                 
	void ViewCoordinate();	

	
	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(COpenGLView)
protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL
	
	// Implementation
protected:
	COpenGLView(); // protected constructor used by dynamic creation
	virtual ~COpenGLView();

	DECLARE_DYNCREATE(COpenGLView)

	
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	
	// Generated message map functions
protected:
	//{{AFX_MSG(COpenGLView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined _OPENGLVIEW_H_
